/*
 * Copyright (c) 2008, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.carbon.registry.core.utils;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.axiom.om.OMElement;
import org.apache.axiom.om.OMText;
import org.wso2.carbon.registry.core.*;
import org.wso2.carbon.registry.core.jdbc.utils.ServiceConfigUtil;
import org.wso2.carbon.registry.core.session.CurrentSession;
import org.wso2.carbon.registry.core.config.RegistryConfiguration;
import org.wso2.carbon.registry.core.exceptions.RegistryException;
import org.wso2.carbon.registry.core.internal.RegistryCoreServiceComponent;
import org.wso2.carbon.user.core.UserRealm;
import org.wso2.carbon.user.core.UserStoreException;
import org.wso2.carbon.user.core.UserStoreReader;

import javax.servlet.http.HttpServletResponse;
import javax.xml.namespace.QName;
import java.io.*;
import java.util.Iterator;
import java.sql.Timestamp;

public class RegistryUtils {

    public static final Log log = LogFactory.getLog(RegistryUtils.class);

    /**
     * Paths refering to a version of a resource are in the form /c1/c2/r1?v=2. Give a such
     * version path, this method extracts the valid pure path and the version number of the
     * resource. Note that this method should only be used for pure resource paths.
     *
     * @param resourcePath versioned resource path.
     *
     * @return VersionPath object containing the valid resource path and the version number.
     */
    public static VersionedPath getVersionedPath(ResourcePath resourcePath) {

        String versionString = resourcePath.getParameterValue("version");
        long versionNumber = -1;
        if (versionString != null) {
            versionNumber = Long.parseLong(versionString);
        }

        String plainPath = getPureResourcePath(resourcePath.getPath());

        VersionedPath versionedPath = new VersionedPath();
        versionedPath.setPath(plainPath);
        versionedPath.setVersion(versionNumber);

        return versionedPath;
    }

    public static byte[] getByteArray(InputStream inputStream) throws RegistryException {

        if (inputStream == null) {
            String msg = "Could not create memory based content for null input stream.";
            log.error(msg);
            throw new RegistryException(msg);
        }

        ByteArrayOutputStream out = null;
        try {
            out = new ByteArrayOutputStream();
            byte[] contentChunk = new byte[1024];
            int byteCount;
            while ((byteCount = inputStream.read(contentChunk)) != -1) {
                out.write(contentChunk, 0, byteCount);
            }
            out.flush();

            return out.toByteArray();

        } catch (IOException e) {
            String msg = "Failed to write data to byte array input stream. " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg, e);
        } finally {
            try {
                inputStream.close();
                if (out != null) {
                    out.close();
                }
            } catch (IOException e) {
                String msg = "Failed to close streams used for creating memory stream. "
                        + e.getMessage();
                log.error(msg, e);
            }
        }
    }

    public static InputStream getMemoryStream(InputStream inputStream) throws RegistryException {
        return new ByteArrayInputStream(getByteArray(inputStream));
    }

    /**
     * All "valid" paths pure resources should be in the form /c1/c2/r1. That is they should start
     * with "/" and should not end with "/". Given a path of a pure resource, this method prepares
     * the valid path for that path.
     *
     * @param resourcePath Path of a pure resource.
     *
     * @return Valid path of the pure resource.
     */
    public static String getPureResourcePath(String resourcePath) {
        if (resourcePath == null) return null;

        String preparedPath = resourcePath;
        if (preparedPath.equals(RegistryConstants.ROOT_PATH)) {
            return preparedPath;
        } else {
            if (!preparedPath.startsWith(RegistryConstants.ROOT_PATH)) {
                preparedPath = RegistryConstants.ROOT_PATH + preparedPath;
            }
            if (preparedPath.endsWith(RegistryConstants.PATH_SEPARATOR)) {
                preparedPath = preparedPath.substring(0, preparedPath.length() - 1);
            }
            if (preparedPath.contains("//")) {
                preparedPath = preparedPath.replace("//", "/");
            }
        }

        return preparedPath;
    }

    public static String getParentPath(String resourcePath) {
        if (resourcePath == null) return null;

        String parentPath;
        if (resourcePath.equals(RegistryConstants.ROOT_PATH)) {
            parentPath = null;
        } else {
            String formattedPath = resourcePath;
            if (resourcePath.endsWith(RegistryConstants.PATH_SEPARATOR)) {
                formattedPath = resourcePath.substring(
                        0, resourcePath.length() - RegistryConstants.PATH_SEPARATOR.length());
            }

            if (formattedPath.lastIndexOf(RegistryConstants.PATH_SEPARATOR) == 0) {
                parentPath = RegistryConstants.ROOT_PATH;
            } else {
                parentPath = formattedPath.substring(
                        0, formattedPath.lastIndexOf(RegistryConstants.PATH_SEPARATOR));
            }
        }

        return parentPath;
    }

    public static String getResourceName(String resourcePath) {
        String resourceName;
        if (resourcePath.equals(RegistryConstants.ROOT_PATH)) {
            resourceName = RegistryConstants.ROOT_PATH;

        } else {

            String formattedPath = resourcePath;
            if (resourcePath.endsWith(RegistryConstants.PATH_SEPARATOR)) {
                formattedPath = resourcePath.substring(
                        0, resourcePath.length() - RegistryConstants.PATH_SEPARATOR.length());
            }

            if (formattedPath.lastIndexOf(RegistryConstants.PATH_SEPARATOR) == 0) {
                resourceName = formattedPath.substring(1, formattedPath.length());
            } else {
                resourceName = formattedPath.substring(
                        formattedPath.lastIndexOf(RegistryConstants.PATH_SEPARATOR) + 1,
                        formattedPath.length());
            }
        }

        return resourceName;
    }

    public static void redirect(HttpServletResponse response, String url) {

        try {
            response.sendRedirect(url);
        } catch (IOException e) {
            String msg = "Failed to redirect to the URL " + url + ". \nCaused by " +
                    e.getMessage();
            log.error(msg, e);
        }
    }

    /**
     * Prepare the given path as a general resource path to be used in the registry. General
     * resource paths may refer to pure resources or virtual resources.
     *
     * @param rawPath Raw path to be prepared
     * @return Prepared general resource path
     */
    public static String prepareGeneralPath(String rawPath) {
        String path = rawPath;

        // path should always start with "/"
        if (!rawPath.startsWith(RegistryConstants.ROOT_PATH)) {
            path = RegistryConstants.ROOT_PATH + rawPath;
        }

        return path;
    }

    public static boolean hasAdminAuthorizations(String userName, UserRealm userRealm)
            throws RegistryException {

        try {
            UserStoreReader userStoreReader = userRealm.getUserStoreReader();
            RegistryConfiguration regConfig = RegistryCoreServiceComponent.getRegistryConfig();
            String systemuser = regConfig.getValue(RegistryConfiguration.SYSTEM_USER_NAME);   
            
            if (systemuser.equals(userName)) {
                return true;
            }

            if (RegistryConstants.ADMIN_USER.equals(userName)) {
                return true;
            }

            String[] roles = userStoreReader.getUserRoles(userName);
            if (containsString(RegistryConstants.ADMIN_ROLE, roles)) {
                return true;
            }

        } catch (UserStoreException e) {

            String msg = "Failed to check authorization level of user " +
                    userName + ". Caused by: " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg, e);
        }

        return false;
    }

    public static String removeUISuffixes(String path) {

        if (path.endsWith(RegistryConstants.VIEW_ACTION)) {
            return path.substring(0, path.length() - RegistryConstants.VIEW_ACTION.length());

        } else if (path.endsWith(RegistryConstants.EDIT_ACTION)) {
            return path.substring(0, path.length() - RegistryConstants.EDIT_ACTION.length());
        }

        return path;
    }

    public static boolean containsString(String value, String[] array) {
        boolean found = false;

        for (String anArray : array) {
            if (anArray.equals(value)) {
                found = true;
            }
        }

        return found;
    }

    public static boolean containsAsSubstring(String value, String[] array) {
        boolean found = false;

        for (String anArray : array) {
            if (anArray.indexOf(value) != -1) {
                found = true;
            }
        }

        return found;
    }

    public static OMElement removeAuthorUpdatorFromDump(OMElement root) {
        Iterator children = root.getChildren();

        String isCollectionString = root.getAttributeValue(new QName("isCollection"));
        boolean isCollection = isCollectionString.equals("true");

        OMElement creatorEle = null;
        OMElement createdTimeEle = null;
        OMElement lastUpdaterEle = null;
        OMElement lastModifiedTimeEle = null;

        OMElement childsEle = null;
        while (children.hasNext()) {
            Object nextChild = children.next();
            if (!(nextChild instanceof OMElement)) {
                continue;
            }
            OMElement child = (OMElement)nextChild;
            String localName = child.getLocalName();
            // creator
            if (localName.equals("creator")) {
                creatorEle = child;
            }
            // createdTime
            else if (localName.equals("createdTime")) {
                createdTimeEle = child;
            }
            // setLastUpdater
            else if (localName.equals("lastUpdater")) {
                lastUpdaterEle = child;
            }
            // LastModified
            else if (localName.equals("lastModified")) {
                lastModifiedTimeEle = child;
            }
            else if (localName.equals("childs")) {
                childsEle = child;
            }
        }

        // removing the childs
        if (creatorEle != null) {
            creatorEle.detach();
            creatorEle = null;
        }
        if (createdTimeEle != null) {
            createdTimeEle.detach();
            createdTimeEle = null;
        }
        if (lastUpdaterEle != null) {
            lastUpdaterEle.detach();
            lastUpdaterEle = null;
        }
        if (lastModifiedTimeEle != null) {
            lastModifiedTimeEle.detach();
            lastModifiedTimeEle = null;
        }

        // call this for all the childs
        if (isCollection && childsEle != null) {
            Iterator childs = childsEle.getChildren();

            while(childs.hasNext()) {
                Object nextChild = childs.next();
                if (!(nextChild instanceof OMElement)) {
                    continue;
                }
                OMElement childE = (OMElement)nextChild;
                removeAuthorUpdatorFromDump(childE);
            }
        }

        return root;
    }

    public static void addSystemCollection(Registry registry, UserRealm systemUserRealm) throws RegistryException {

        RegistryConfiguration regConfig = RegistryCoreServiceComponent.getRegistryConfig();
        String username = regConfig.getValue(RegistryConfiguration.SYSTEM_USER_NAME);

        CurrentSession.setUser(username);
        CurrentSession.setRealm(systemUserRealm);

        if (!registry.resourceExists(RegistryConstants.SYSTEM_PATH)) {

            CollectionImpl systemCollection = (CollectionImpl) registry.newCollection();
            String systemDescription = "System collection of the Registry. This collection is used to store " +
                    "the resources required by the Registry server. It is not recommended to do any modification " +
                    "to this collection.";
            systemCollection.setDescription(systemDescription);
            registry.put(RegistryConstants.SYSTEM_PATH, systemCollection);
            String systemCollectionID = systemCollection.getId();
            systemCollection.discard();

            AuthorizationUtils.setAdminAuthorizations(RegistryConstants.SYSTEM_PATH, systemUserRealm);
        }

        CurrentSession.removeUser();
        CurrentSession.removeRealm();
    }

    public static void addUserProfileCollection(Registry registry, UserRealm systemUserRealm, String profilesPath)throws RegistryException{

        RegistryConfiguration regConfig = RegistryCoreServiceComponent.getRegistryConfig();
        String username = regConfig.getValue(RegistryConfiguration.SYSTEM_USER_NAME);

        CurrentSession.setUser(username);
        CurrentSession.setRealm(systemUserRealm);

        if (!registry.resourceExists(profilesPath)) {

            CollectionImpl systemCollection = (CollectionImpl) registry.newCollection();
            String systemDescription = "System collection of the Registry Profiles Storing";
            systemCollection.setDescription(systemDescription);
            registry.put(profilesPath, systemCollection);
        }

        CurrentSession.removeUser();
        CurrentSession.removeRealm();
    }

    public static void addServiceStoreCollection(Registry registry, UserRealm systemUserRealm, String servicePath)throws RegistryException{
        RegistryConfiguration regConfig = RegistryCoreServiceComponent.getRegistryConfig();
        String username = regConfig.getValue(RegistryConfiguration.SYSTEM_USER_NAME);

        CurrentSession.setUser(username);
        CurrentSession.setRealm(systemUserRealm);

        if (!registry.resourceExists(servicePath)) {

            CollectionImpl systemCollection = (CollectionImpl) registry.newCollection();
            String systemDescription = "Collection which contains all the Service information";
            systemCollection.setDescription(systemDescription);
            registry.put(servicePath, systemCollection);
        }

        CurrentSession.removeUser();
        CurrentSession.removeRealm();
    }

    public static void addMountCollection(Registry registry, UserRealm systemUserRealm) throws RegistryException {

        RegistryConfiguration regConfig = RegistryCoreServiceComponent.getRegistryConfig();
        String username = regConfig.getValue(RegistryConfiguration.SYSTEM_USER_NAME);

        CurrentSession.setUser(username);
        CurrentSession.setRealm(systemUserRealm);

        if (!registry.resourceExists(RegistryConstants.SYSTEM_PATH+"/mount")) {
            Collection mountCollection = (CollectionImpl) registry.newCollection();
            String description = "Mount collection stores details of mount points of the registry.";
            mountCollection.setDescription(description);
            registry.put(RegistryConstants.SYSTEM_PATH+"/mount", mountCollection);

            AuthorizationUtils.setAdminAuthorizations(RegistryConstants.SYSTEM_PATH+"/mount", systemUserRealm);
        }

        CurrentSession.removeUser();
        CurrentSession.removeRealm();
    }

    public static void registerMountPoints(Registry registry, UserRealm systemUserRealm) throws RegistryException {

        RegistryConfiguration regConfig = RegistryCoreServiceComponent.getRegistryConfig();
        String username = regConfig.getValue(RegistryConfiguration.SYSTEM_USER_NAME);

        CurrentSession.setUser(username);
        CurrentSession.setRealm(systemUserRealm);

        CollectionImpl mountCollection = (CollectionImpl) registry.get(RegistryConstants.SYSTEM_PATH + "/mount");
        String[] mountPoints = mountCollection.getChildren();
        Resource mountPoint;
        for (int i = 0; i < mountPoints.length; i++) {
            mountPoint = registry.get(mountPoints[i]);
            String path = mountPoint.getProperty("path");
            String target = mountPoint.getProperty("target");
            String targetSubPath = mountPoint.getProperty("subPath");

            try {
                if (targetSubPath != null) {
                    registry.createLink(path, target, targetSubPath);
                }
                registry.createLink(path, target);
            } catch (RegistryException e) {
                log.warn("Couldn't do mounting for "+ target + e.getMessage());
                continue;
            }
        }

        CurrentSession.removeUser();
        CurrentSession.removeRealm();
    }

    public static void addServiceConfigResources(Registry registry, UserRealm systemUserRealm)throws RegistryException{
        RegistryConfiguration regConfig = RegistryCoreServiceComponent.getRegistryConfig();
        String username = regConfig.getValue(RegistryConfiguration.SYSTEM_USER_NAME);
        CurrentSession.setUser(username);
        CurrentSession.setRealm(systemUserRealm);
        String[] names = getServiceConfigPaths();
        String[] omtext = ServiceConfigUtil.getAllConfigOMElementText();
        for(int i=0;i<names.length;i++){
            if(!registry.resourceExists(names[i])){
                ResourceImpl configresource = (ResourceImpl) registry.newResource();
                configresource.setContent(omtext[i].getBytes());
                registry.put(names[i],configresource);
            }
        }
        CurrentSession.removeUser();
        CurrentSession.removeRealm();
    }

    private static String[] getServiceConfigPaths(){
        String[] names = RegistryConstants.GOVERNANCE_SERVICE_CONFIG.split(";");
        String[] configpaths = new String[names.length];
        for(int i=0;i<names.length;i++){
            configpaths[i] = RegistryConstants.GOVERNANCE_SERVICES_CONFIG_PATH + names[i];
        }
        return configpaths;
    }
}
